package com.softwarepearls.apps.hardware.raspberry.artekit.shiftreg;

import com.pi4j.io.gpio.GpioPinDigitalOutput;

/**
 * Texas Instruments 8-Bit Shift Register.
 * <P>
 * <P>
 * Requires three control lines:
 * <UL>
 * <LI>SER (the shifted-in bit)
 * <LI>SRCLK (the clock signal ordering a bit to shift in)
 * <LI>RCLK (the value latch clock signal ordering the shifted bits to be put on
 * the output)
 * </UL>
 */
public final class SN74LV595 {

	private final GpioPinDigitalOutput ser;
	private final GpioPinDigitalOutput rclk;
	private final GpioPinDigitalOutput srclk;

	/**
	 * Constructor.
	 * 
	 * @param ser GPIO pin wired to the SER input
	 * @param srclk GPIO pin wired to the SRCLK (SER CLOCK) input
	 * @param rclk GPIO pin wired to the RCLK (LATCH) input
	 */
	public SN74LV595(final GpioPinDigitalOutput ser, final GpioPinDigitalOutput srclk, final GpioPinDigitalOutput rclk) {

		this.ser = ser;
		this.srclk = srclk;
		this.rclk = rclk;

		ser.low();
		srclk.low();
		rclk.low();
	}

	/**
	 * Shift the given byte (8 bits) out to the register, LSb first. This does
	 * NOT latch the bits.
	 * 
	 * @param b any byte
	 */
	public void shiftOutLeastSignificantBitsFirst(final byte b) {

		byte bits = b;
		for (int i = 0; i < 8; i++) {

			// set SER
			if ((bits & 0x01) == 1) {
				ser.high();
			} else {
				ser.low();
			}
			srclk.high();
			srclk.low();

			bits >>= 1;
		}
	}

	/**
	 * Shift the given byte (8 bits) out to the register, MSb first. This does
	 * NOT latch the bits.
	 * 
	 * @param b any byte
	 */
	public void shiftOutMostSignificantBitsFirst(final byte b) {

		byte bits = b;
		for (int i = 0; i < 8; i++) {

			// set SER
			if ((bits & 0x80) != 0) {
				ser.high();
			} else {
				ser.low();
			}
			srclk.high();
			srclk.low();

			bits <<= 1;
		}
	}

	/**
	 * Latch whatever bit sequence has been shifted in.
	 */
	public void latchBits() {

		rclk.high();
		rclk.low();
	}

	@Override
	public String toString() {

		return getClass().getSimpleName() + "[SER:" + ser + ", SRCLK:" + srclk + ". RCLK:" + rclk + "]";
	}
}
